﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Configuration {
  using HeuristicLab.Core;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Expressions;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  public class EnabledExpressionsChangedEventArgs : EventArgs {
    public IEnumerable<string> AddedExpressions { get; private set; }
    public IEnumerable<string> RemovedExpressions { get; private set; }

    public EnabledExpressionsChangedEventArgs(IEnumerable<string> addedExpressions, IEnumerable<string> removedExpressions) {
      AddedExpressions = addedExpressions;
      RemovedExpressions = removedExpressions;
    }
  }

  public interface IEnabledExpressionsConfiguration : INamedItem {
    event EventHandler<EnabledExpressionsChangedEventArgs> EnabledExpressionsChanged;

    IReadOnlyList<string> EnabledExpressions { get; }

    void EnableStack(StackTypes types, bool enableDependencies = false);
    void DisableStack(StackTypes type, bool enableDepenencies = false);
    void SetStack(StackTypes type, bool state, bool setDependencies = false);
    void EnableExpressionOfStack(StackTypes types);
    void DisableExpressionsOfStack(StackTypes types);
    void EnableExpression(string name);
    void DisableExpression(string name);
    void SetExpression(string name, bool state);
    void SetExpression<T>(bool state) where T : Expression;
    void EnableExpression<T>() where T : Expression;
    void DisableExpression<T>() where T : Expression;
  }
}
